   processor 6502
   include vcs.h
;
;
; background color is subtracted by #$4E
;

DEBUGMODE = 0
;============================================================================
; C O N S T A N T S
;============================================================================
NUMGRP         =  7              ; number of graphics/platform levels
GRPHEIGHT      =  20             ; height of each player
MAXSELECTLEVEL =  7              ; maximum possible selected level

XMIN           =  8              ; minimum x value for players
XMAX           =  158            ; maximum x value for players

;------------------------------------------------------------------------------
; NTSC frame constants
;
NTSCVBLANKTIME    =  $2B
NTSCFRAMETIME     =  $10
NTSCOVERSCANTIME  =  $23
NTSCKERNELLINES	=	$C0

;------------------------------------------------------------------------------
; PAL frame constants
;
PALVBLANKTIME     =  $35
PALFRAMETIME      =  $12
PALOVERSCANTIME   =  $2A
PALKERNELLINES		=	$E4

;----------------------------------------------------------------------------
; game state options
;
GAMERUNNING    =  #%00000000
TITLESCREEN    =  #%10000000
GAMEPAUSED     =  #%01000000

;----------------------------------------------------------------------------
; color constants
;
BLACK          =  $00
LTBLUE         =  $78
LADDER_COLOR	=	$9A
BLUE           =  $86
PEACH				=	$3C
;GREEN				= %11101000
GREEN = %11011111


;YELLOW          = %00100000
;RED             = %01000000
;ORANGE          = %00100000
;        ENDIF
;
;DARK            = %0000
;LIGHT           = %1111
;MEDIUM          = %1000


;============================================================================
; M A C R O S
;============================================================================

;
; courtesy of Thomas Jentzsch
;
  MAC SLEEP
    IF {1} = 1
      ECHO "ERROR: SLEEP 1 not allowed !"
      END
    ENDIF
    IF {1} & 1
        bit $00
      REPEAT ({1}-3)/2
        nop
      REPEND
    ELSE
      REPEAT ({1})/2
        nop
      REPEND
    ENDIF
  ENDM
  
;============================================================================
; Z P - V A R I A B L E S
;============================================================================
   SEG.U vars
   org $80
   
; There is only 128 bytes of RAM available to the programmer. Locations
; $80 - $FF are available for RAM use.
;

; The random bytes are used to produce random number for the game. These
; variables and random routine are courtesy of Andrew Davie. He used it in
; his 'Qb' game. The source for 'Qb' can be found at http://www.atari2600.org.
;
random               ds 2

temp                 ds 1
tempStack            ds 1
tvType               ds 1           ; 0 = NTSC, 1 = PAL
selectedLevel        ds 1           ; the level number the player selected
gameState            ds 1
backgroundColor      ds 1
platformColor        ds 1
ladderColor          ds 1
verPosP0             ds 1           ; vertical position of climber
horPosP0             ds 1           ; horizontal position of climber
attrackMode          ds 1
horPosP1             ds NUMGRP
horPosP1_FC          ds NUMGRP
startCount           ds NUMGRP      ; count down for obstacle to move
playerMotion         ds 1           ; speed of players (increases by level)
groupCount           ds 1           ; keeps track of which group of obstacles
                                    ; is being processed during the kernel
level                ds 1           ; current level being played
lives                ds 1           ; number of lives available to the player

gameClock            ds 1           ; updated each frame
selectDebounce       ds 1           ; debounce flag for the select switch
levelPointer         ds 2           ; pointer to number data

playerPointer        ds 2
colorPointer			ds 2
graphicsPointers     ds 12
playerGraphics       ds GRPHEIGHT
playerColors         ds GRPHEIGHT
playerGraphic        ds 1
playerColor          ds 1
playerGraphicsTemp   ds 1
playerColorsTemp     ds 1

L06DB						ds 1
L06DC						ds	1
L06DE						ds 1
;
; show how many bytes used for RAM after compiling
;
   echo *-$80, " bytes of zero page used"

   SEG code
   org $F000
SkipDrawGameKernelJump
   lda #$00                      ;[21]+2
   sta playerGraphic					;[23]+4
   jmp LoadBufferedPlayerGraphics;[31]+3		=	34
      
SkipDrawBufferedPlayerGraphics
   lda #$00                      ;[43]+2
   sta playerGraphicsTemp   	   ;[45]+4
   jmp GameKernelHMP1Set			;[55]+3		=	58
   
PositionPlayerRoutine1
   sta WSYNC
;------------------------------------------------------------------------------
   sta HMOVE                     ;[0]+3
   lda playerGraphic             ;[3]+3
   sta.w GRP0                    ;[6]+4
   lda playerColor               ;[10]+3
   sta.w COLUP0                  ;[13]+4
   sta RESP0                     ;[17]+3     =  20
   lda temp
   sta HMP1
   jmp LastPlatformScanline
  
GameKernelStep
	sta WSYNC
	sta HMOVE
   sta COLUP0							;[3]+3
   lda playerGraphic		         ;[6]+3
   sta GRP0                      ;[9]+3
  
   jmp GameKernelJump
	
GameKernel
   sta WSYNC
;------------------------------------------------------------------------------
   sta HMOVE                     ;[0]+3
   sta COLUP0							;[3]+3
   lda playerGraphic		         ;[6]+3
   sta GRP0                      ;[9]+3
   lda platformColor					;[12]+3
   sta COLUBK                    ;[15]+3
   lda #$00								;[18]+3
   sta GRP1				 				;[21]+3	=	24	*HBLANK*

GameKernelJump
   jsr PrepareNextPlayerGraphic	;[24]+6	= 30 + 28 = 58
   ldx groupCount						;[58]+3
   
	sta WSYNC
;------------------------------------------------------------------------------
	sta HMOVE							;[0]+3
	sta COLUP0							;[3]+3
	lda playerGraphic					;[6]+3
	sta GRP0								;[9]+3

   iny                           ;[12]+2   
   bmi SkipDrawGameKernelJump		;[14]+2/3
   cpy #GRPHEIGHT+1              ;[16]+2
   bpl SkipDrawGameKernelJump		;[18]+2/3

	lda playerGraphics,y				;[20]+4
	sta playerGraphic					;[24]+3
	lda playerColors,y				;[27]+4
	sta playerColor					;[31]+3
	
LoadBufferedPlayerGraphics
   iny                           ;[34]+2
   bmi SkipDrawBufferedPlayerGraphics       ;[36]+2/3
   cpy #GRPHEIGHT+1              ;[38]+2
   bpl SkipDrawBufferedPlayerGraphics      ;[40]+2/3
	
	lda playerGraphics,y          ;[42]+4
   sta playerGraphicsTemp        ;[46]+3
   lda playerColors,y            ;[49]+4
   sta playerColorsTemp       	;[53]+3
      
GameKernelHMP1Set
	lda horPosP1_FC,x					;[56]+4
	sta temp 							;[60]+3
	and #$0F								;[63]+2
	tax									;[65]+2
	beq PositionPlayerRoutine1		;[67]+2/3

	dex

StartPlatformScanLine
	sta WSYNC
;------------------------------------------------------------------------------
   sta HMOVE                     ;[0]+3
   lda playerGraphic             ;[3]+3
   sta GRP0								;[6]+3
   lda playerColor               ;[9]+3
   sta COLUP0	                  ;[12]+3
   lda temp								;[15]+3
;	dex
PositionPlayerLoop
   dex									;[18]+2
   bpl PositionPlayerLoop			;[20]+2/3
   sta RESP1							;[67]+3
   sta HMP1
   
LastPlatformScanline
   sta WSYNC
;------------------------------------------------------------------------------
	sta HMOVE                     ;[0]+3
	lda playerGraphicsTemp			;[3]+3
	sta GRP0								;[6]+3
	lda playerColorsTemp				;[9]+3
	sta COLUP0							;[12]+3
	lda #$00								;[15]+2
	sta HMP1								;[17]+3
	sta playerGraphic					;[20]+3
	
	iny                           ;[26]+2   
	bmi DrawPlayerGroups				;[28]+2/3
	cpy #GRPHEIGHT+1              ;[30]+2
	bpl DrawPlayerGroups				;[32]+2/3
	
	lda playerGraphics,y				;[34]+4
	sta playerGraphic					;[38]+3
	lda playerColors,y				;[41]+4
	sta playerColor					;[45]+3
	
DrawPlayerGroups
   ldx groupCount           		;[48]+2
   
   lda backgroundColor				;[50]+3
   sta WSYNC
;------------------------------------------------------------------------------
   sta HMOVE                     ;[0]+3
   sta COLUBK                    ;[3]+3
   lda playerGraphic             ;[6]+3
   sta GRP0                      ;[9]+3
   lda playerColor               ;[12]+3
	sta COLUP0							;[15]+3
   lda #$03								;[18]+2
   sta GRP1                      ;[20]+3
   lda #$00								;[23]+2
   sta playerGraphic					;[25]+3
   
   iny                           ;[31]+2   
	bmi PrepareGroupKernel  		;[33]+2/3
	cpy #GRPHEIGHT+1              ;[35]+2
	bpl PrepareGroupKernel			;[37]+2/3
	
	lda playerGraphics,y				;[39]+4
	sta playerGraphic					;[43]+3
	lda playerColors,y				;[46]+4
	sta playerColor					;[50]+3
   
PrepareGroupKernel
	txa									;[53]+2
	dex									;[55]+2
	stx groupCount						;[57]+3

	cmp #NUMGRP-1						;[60]+2
	beq ZeroGroup						;[62]+2/3
	lsr									;[64]+2
	bcs EvenGroup						;[66]+2/3
	
	ldx #GRPHEIGHT						;[67]+3
OddGroupLoop
   sta WSYNC
------------------------------------------------------------------------------
   sta HMOVE                     ;[0]+3
   lda playerColor					;[3]+3
   sta COLUP0							;[6]+3
   lda playerGraphic             ;[9]+3
   sta GRP0                      ;[12]+3

   lda OddLadderTablePF1,x			;[15]+4
   sta PF1								;[19]+3
   
   lda OddLadderTablePF2,x			;[22]+4
   sta PF2								;[26]+3

;
; NOTE: Remove jmp - place code here ~ 30 cycles would be used
;
   jmp PrepareNextPlayerGraphic3 ;[29]+3	= 32 + 25 = 57
ContinueOddGroupKernel
   dex                           ;[57]+2
   bpl OddGroupLoop              ;[59]+2/3
	jmp IncrementGroupCount			;[61]+3

PrepareNextPlayerGraphic3
   iny                           ;[29]+2				[0]+2
   bmi SkipDrawNextPlayerGraphic3;[31]+2/3			[2]+2/3
   cpy #GRPHEIGHT+1              ;[33]+2				[4]+2
   bpl SkipDrawNextPlayerGraphic3;[35]+2/3			[6]+2
   lda playerGraphics,y          ;[37]+4				[8]+4
   sta playerGraphic             ;[41]+3				[12]+3
   lda playerColors,y				;[44]+4				[15]+4
   sta playerColor               ;[48]+3				[19]+3
   jmp ContinueOddGroupKernel    ;[51]+3				[22]+3     = 40

SkipDrawNextPlayerGraphic3
   lda #$00								;[38]+2
   sta playerGraphic					;[40]+3
   jmp ContinueOddGroupKernel		;[51]+3

ZeroGroup
	ldx #GRPHEIGHT
ZeroGroupLoop
   sta WSYNC
;------------------------------------------------------------------------------
   sta HMOVE                     ;[0]+3
   lda playerGraphic             ;[3]+3
   sta GRP0                      ;[6]+3
   lda playerColor               ;[9]+3
   sta COLUP0							;[12]+3
   lda #$00								;[15]+2
   sta PF0								;[17]+3
   sta PF1								;[20]+3
   sta PF2								;[23]+3
   sta playerGraphic					;[26]+3
   
   iny                           ;[32]+2
	bmi DecrementX 					;[34]+2/3
	cpy #GRPHEIGHT+1              ;[37]+2
	bpl DecrementX 					;[39]+2/3
	lda playerGraphics,y          ;[41]+4
	sta playerGraphic             ;[45]+3
	lda playerColors,y            ;[47]+4
   sta playerColor               ;[51]+3
  
DecrementX   
	dex                           ;[54]+2
   bpl ZeroGroupLoop             ;[56]+2/3

	jmp IncrementGroupCount			;[58]+3

EvenGroup
	ldx #GRPHEIGHT						;[67]+3
EvenGroupLoop
   sta WSYNC
;------------------------------------------------------------------------------
   sta HMOVE                     ;[0]+3
   lda playerGraphic             ;[3]+3
   sta GRP0                      ;[6]+3
   lda playerColor               ;[9]+3
   sta COLUP0							;[12]+3
   lda EvenLadderTablePF0,x		;[15]+4
   sta PF0								;[19]+3
   sta PF1								;[22]+3
   
   lda EvenLadderTablePF2,x		;[25]+4
   sta PF2								;[29]+3

;
; NOTE: Remove jmp - place code here ~ 30 cycles would be used
;
   jmp PrepareNextPlayerGraphic2	;[32]+6	= 38 + 28 = 66
ContinueKernel
   dex                           ;[66]+2
   bpl EvenGroupLoop             ;[68]+2/3
	
IncrementGroupCount
	ldx groupCount						;[70]+3
	bmi OutKernel						;[73]+2/3

	jmp GameKernel
  
OutKernel
   sta WSYNC                  ; Finish current scanline
;------------------------------------------------------------------------------
   sta HMOVE
	ldx platformColor
   stx COLUBK
   lda #$00                   ; Avoid bleeding of Player1
   sta GRP1
   sta GRP0
   sta PF0
   sta PF1
   sta PF2
   ldx tvType
   lda KernelScanlines,x
   sec
   sbc #[[4+GRPHEIGHT]*NUMGRP]+1
   tax

GameKernelLoop
   sta WSYNC
   sta HMOVE
   dex
   bne GameKernelLoop
   rts

KernelScanlines
	.byte #NTSCKERNELLINES
	.byte #PALKERNELLINES
	
PrepareNextPlayerGraphic
   iny                           ;[18]+2     [0]+2
   bmi SkipDrawNextPlayerGraphic ;           [2]+2/3
   cpy #GRPHEIGHT+1              ;[22]+2     [4]+2
   bpl SkipDrawNextPlayerGraphic ;           [6]+2/3
   lda playerGraphics,y          ;[26]+4     [8]+4
   sta playerGraphic             ;[30]+3     [12]+3
   lda playerColors,y            ;[33]+4     [15]+4
   sta playerColor               ;[37]+3     [19]+3
   rts                           ;[40]+6     [22]+6
   
SkipDrawNextPlayerGraphic
   lda #$00
   sta playerGraphic
   rts
   
PrepareNextPlayerGraphic2
   iny                           ;[15]+2				[0]+2
   bmi SkipDrawNextPlayerGraphic2;[17]+2/3			[2]+2/3
   cpy #GRPHEIGHT+1              ;[19]+2				[4]+2
   bpl SkipDrawNextPlayerGraphic2;[21]+2/3			[6]+2
   lda playerGraphics,y          ;[23]+4				[8]+4
   sta playerGraphic             ;[27]+3				[12]+3
   lda playerColors,y				;[30]+4				[15]+4
   sta playerColor               ;[34]+3				[19]+3
   jmp ContinueKernel            ;[37]+3     = 40
   
SkipDrawNextPlayerGraphic2
   lda #$00
   sta playerGraphic
   jmp ContinueKernel

   echo *-$F000, " BYTES USED FOR GAME KERNEL"
Start
;
; The 2600 powers up in a completely random state, except for the PC which is
; set to the location at $FFC. Therefore the first thing that must be done is
; to set up everything inside the 6502. Location $80-$81 will be avoided
; during the clear RAM routine to keep a random number as the seed for the
; random routine.
;
   sei                           ; The 6507 has no interrupt capability (one
                                 ; of the differences between the 6507 and
                                 ; 6502) so the interrupts are disabled here
                                 ; only for clarity
   cld                           ; Clear the BCD bit
   ldx #$FF
   txs                           ; Set the stack to the beginning
   
   lda #$00
   
;
; The 2600 has 128 bytes of RAM. This routine will not clear VSYNC or
; locations $80-$81.
;
ClearRAM
   cpx #random                   ; is x < #$80
   bcc ClearRAMRegister          ; if so then clear the RAM location
   cpx #random+2                 ; is x >= #$82
   bcs ClearRAMRegister          ; if so then clear the RAM location
   jmp NextRAMLocation           ; #$80 > x > #$81 so skip to next location
ClearRAMRegister
   sta VSYNC,x
NextRAMLocation
   dex
   bne ClearRAM
   
   lda #$01
   sta selectedLevel
   
   jsr InitializeGame
   
;--------------------------------------------------------------------MainLoop
;
; This is the main program loop. In this loop the program takes care of
; the vertical blanking, checking console switches, performing game
; calculations, and drawing the screen. This loop continues forever until the
; game is turned off.
;
MainLoop
   jsr VerticalBlank
   jsr CheckConsoleSwitches
   jsr GameCalculations
   jsr DrawScreen
   jsr Overscan
   jmp MainLoop
   
;---------------------------------------------------------------VerticalBlank
;
; This routine will take care of vertical blank house keeping. Vertical 
; blanking starts a new television frame. Each frame starts with 3 vertical
; sync lines. These signal to the television to start a new frame. During
; the vertical sync time the right difficulty setting will be checked to
; determine if the game is for an NTSC or PAL setting. This way the vertical
; blank wait time can be set appropriately.
;
VerticalBlank
   ldy #$00
   ldx #$02
      
;   sta WSYNC
   stx VSYNC

	sta WSYNC
	sta WSYNC
	
   lda SWCHB                     ; read the console switch value
   rol                           ; right difficutly setting in carry bit
   rol                           ; right difficulty setting in bit 0
   and #%00000001                ; mask the unneeded bits
   sta tvType                    ; tvType is now set to 0 or 1
   tax
   inc gameClock	
	
	lda VBlankTime,x
	sta TIM64T
	sta WSYNC
	sty VSYNC
   rts
   
VBlankTime
   .byte #NTSCVBLANKTIME
   .byte #PALVBLANKTIME
   
;--------------------------------------------------------CheckConsoleSwitches
;
; The Reset and Select switches will be checked for use.
; Game Reset will restart the game at the player's selected level.
; Game Select will hold two functions. While on the game menu the game select
; will be used to choose the starting game level. During game play the game
; select will be used to pause the game action.
;
CheckConsoleSwitches
   lda SWCHB
   lsr                           ; reset value now in carry bit
   bcs CheckForSelect            ; reset not pressed so check select switch
   
   lda selectedLevel
   sta level
   
   jsr NewLevel
   jsr InitializeGame
   rts
   
CheckForSelect
   tax
   lda gameClock
   and #$3F
   bne CheckSelectSwitch
   
   sta selectDebounce            ; Reset the select debounce flag. This is 
                                 ; what keeps incrementing the selection if
                                 ; you hold select down.
CheckSelectSwitch
   txa                           ; remember we put the switch value in x
   lsr                           ; select value now in carry bit
   bcc SelectSwitchDown
   sta selectDebounce
   beq SelectSwitchDown
   rts                           ; selectDebounce ~= 0 so we leave
   
SelectSwitchDown
   bit selectDebounce
   bmi ReturnFromConsoleSwitches 
   lda #$FF
   sta selectDebounce            ; show the select switch has been down
   
   bit gameState
   bmi IncrementGameSelection
   
   lda gameState
   eor #GAMEPAUSED
   sta gameState
   rts
   
IncrementGameSelection
   lda selectedLevel             ; get the selected level number
   cmp #MAXSELECTLEVEL           ; if it's less than the max allowed
   bcc IncrementSelectedLevel    ; increment the level number 
   lda #$00
   sta selectedLevel
IncrementSelectedLevel
   inc selectedLevel
   
ReturnFromConsoleSwitches
   rts
   
NewLevel
   lda level
   asl                        ; multiply the level by 2
   adc tvType
   sta playerMotion
   
   ldy #XMIN
   ldx #NUMGRP-1
InitializePlayerStartCount   
   
   jsr RandomByte             ; get a random value for the players next
   sta startCount,x           ; start count value
   ora random+1

   sty horPosP1,x
   jsr RandomByte
   dex
   bpl InitializePlayerStartCount
   rts
   
InitializeGame
   lda #$05
   sta lives
   
   lda #GAMERUNNING
   sta gameState
   
   ldx #$05
InitializeGameLoop
   lda GameInitializeTable,x
   sta gameState,x
   dex
   bpl InitializeGameLoop
   
   ldx #GRPHEIGHT
PlayerLoop
	lda HorizontalMoveFrame1,x
	sta playerGraphics,x
	
	lda PlayerColor,x
	sta playerColors,x
	dex
	bpl PlayerLoop
      
   rts
   
GameInitializeTable
   .byte #GAMERUNNING,#BLACK,#$3c,#LADDER_COLOR,$32,$96
   
   
;------------------------------------------------------------GameCalculations
;
; The players will be moved in this routine. The sound will be done during
; overscan calculations.
;
GameCalculations
   bit gameState                 ; check the current game state
   bmi TitleCalculations         ; bit 7 is set high for the title
   bvs DoneCalculations          ; bit 6 is set high for pause
   jmp DoGameCalculations

; convert the selectedLevel to the font data so it can be displayed for the
; title screen
;
TitleCalculations
   lda selectedLevel
   asl
   asl
   asl
   clc
   adc #<NumberFonts
   sta levelPointer
   
   lda #$00
   sta NUSIZ0
   adc #>NumberFonts
   sta levelPointer+1
   
   lda #LTBLUE
   sta COLUP0
   sta COLUP1
   
   ldy #$0B
SetTitlePointers
   lda TitleTable,y
   sta graphicsPointers,y
   dey
   bpl SetTitlePointers
   rts
   
TitleTable
   .word TitleData0, TitleData1
   .word TitleData2, TitleData3
   .word TitleData4, BLANK

DoneGameCalculations
	ldy #GRPHEIGHT
LoadPlayerGraphics
	lda (playerPointer),y
	sta playerGraphics,y
	lda (colorPointer),y
	sta playerColors,y
	dey
	bpl LoadPlayerGraphics
	
DoneCalculations
   lda #NUMGRP-1
   sta groupCount
   
;==============================================================DEGUGGING
; this is used to calculate the verPosP0 and horPosP0
; delete this from the final source
;
   ldy #$00
   ldx #$04
DebugLoop
   lda verPosP0,y
   and #$F0
   adc #<NumberFonts
   sta graphicsPointers,x
   lda #>NumberFonts
   sta graphicsPointers,x
   dex
   dex
   lda verPosP0,y
   and #$0F
   
   asl
   asl
   asl
   adc #<NumberFonts
   sta graphicsPointers,x
   lda #>NumberFonts
   sta graphicsPointers,x
   iny
   dex
   dex
   bpl DebugLoop
;============================================================END DEBUGGING   
   lda horPosP0
   jsr ConvertToFC					; a will hold P0 horizontal FC position
   ldx #%00010001
   sta WSYNC
   
   sta temp                      ;[0]+3
   and #$0F                      ;[3]+2
   tay                           ;[5]+2
	lda temp								;[7]+3
   
   SLEEP 3								;[10]+3
   
PositionPlayer0
   dey									;[13]+2
   bpl PositionPlayer0
   sta RESP0
   sta HMP0
   
   sta WSYNC
   sta HMOVE
   
   stx CTRLPF
;   ldx backgroundColor

   ldy ladderColor
;   lda #$3E
	lda #$5C
   sta COLUP1
   
;   stx COLUBK
   sty COLUPF
   lda #$00
   sta HMP0
   
;
; This a modified version of Thomas' skipDraw. y is set during VBLANK as an
; offset for the player. During the kernel y will be checked to see if the
; player needs to be drawn. This increases my kernel size but saves a few
; cycles for buffering the player graphics for the next scan line.
;
   sec
   sbc verPosP0                  ;[8]+3
   adc #GRPHEIGHT                ;[11]+3
   tay
   rts

;
; Move the obstacles horizontally. This routine is used courtesy of
; Piero Caviana. It is based on his PCMSD demo which eventually became
; Oyston. His original PCMSD source can be found at
; http://www.biglist.com/lists/stella/archives/199704/msg00015.html
;
DoGameCalculations
   ldx #NUMGRP-1
MovePlayer1Loop
   lda startCount,x
   beq MovePlayer1               ; only move if the player start count is 0
   
   dec startCount,x
   jmp MoveDone
   
MovePlayer1
   lda horPosP1,x                ; get current x position of player 1
   clc
   adc playerMotion
   sta horPosP1,x
   cmp #XMAX
   bcs GetNextStartCount
   jmp MoveDone
   
GetNextStartCount
   jsr RandomByte
   sta startCount,x
   lda #XMIN
   sta horPosP1,x
   
MoveDone
   lda horPosP1,x
   jsr ConvertToFC
   sta horPosP1_FC,x
   dex
   bpl MovePlayer1Loop
   
;---------------------------------------------------------------CheckJoystick
;
; Check the joystick and move the climber based on the value. This is a one
; player game so the lower bits of SWCHA are ignored. The player is not
; allowed to move diagonal so there is no need to check for combination of
; values.
;
CheckJoystick
   lda SWCHA                     ; get the joystick value
   bmi CheckForLeftMove
   
   lda horPosP0
   clc
   adc playerMotion
   sta horPosP0
   lda #$0F
   sta REFP0
   jmp DoneHorizontalCalculations
   
CheckForLeftMove
   asl
   bmi CheckForDownMove
   
   lda horPosP0
	sec
	sbc playerMotion
	sta horPosP0
	
	lda #$00
	sta REFP0
   jmp DoneHorizontalCalculations
   
CheckForDownMove
   asl
   bmi CheckForUpMove
   
   lda verPosP0
   clc
   adc playerMotion
   sta verPosP0
   jmp DoneVerticalCalculations
   
CheckForUpMove
   asl
   bpl MovePlayerUp
   jmp DoneCalculations
MovePlayerUp
   lda verPosP0
   sec
   sbc playerMotion
   sta verPosP0
   jmp DoneVerticalCalculations
   
DoneVerticalCalculations
	lda #$00
	sta REFP0
	lda gameClock
	lsr
	lsr
	and #$01
	tax
	lda LowVerticalMovementTable,x
	sta playerPointer
	lda HighVerticalMovementTable,x
	sta playerPointer+1
	
	lda #<VerticalColor
	sta colorPointer
	
	lda #>VerticalColor
	sta colorPointer+1	
	jmp DoneGameCalculations
	
HighVerticalMovementTable
	.byte #>VerticalMoveFrame1
	.byte #>VerticalMoveFrame2
	
LowVerticalMovementTable
	.byte #<VerticalMoveFrame1
	.byte #<VerticalMoveFrame2
	
DoneHorizontalCalculations
	lda gameClock
	lsr

	and #$03
	tax
	lda LowHorizontalMovementTable,x
	sta playerPointer
	lda HighHorizontalMovementTable,x
	sta playerPointer+1
	
	lda #<PlayerColor
	sta colorPointer
	
	lda #>PlayerColor
	sta colorPointer+1
	
	jmp DoneGameCalculations
   
HighHorizontalMovementTable
	.byte #>HorizontalMoveFrame1
	.byte #>HorizontalMoveFrame2
	.byte #>HorizontalMoveFrame3
	.byte #>HorizontalMoveFrame1
	
LowHorizontalMovementTable
	.byte #<HorizontalMoveFrame1
	.byte #<HorizontalMoveFrame2
	.byte #<HorizontalMoveFrame3
	.byte #<HorizontalMoveFrame1
	
;-----------------------------------------------------------------ConvertToFC
;
; This subroutine was copied from Air-Sea Battle by Larry Kaplan. This
; converts from a standard x position to a fine course position
;
; Comments from Erik Mooney
; Make the X-coord zero if it's out of range (not between zero and 159).
; Then, it takes the fine and coarse values in base-16 of the number (which
; are simply the high and low nibbles). To convert base-16 FC to base-15 FC,
; you add the F and C values and put that in F; if that's >15, you subtract
; 15 from F and add one to C. (Trust me, this works mathematically, though
; it's difficult to explain). The rest of the code cleans up and puts the F
; and C values together into one register.
;
ConvertToFC
   sta temp
   bpl LF34B
   
   cmp #XMAX+8
   bcc LF34B
   lda #$00
   sta temp
;
; Here, the X-coordinate has been converted to zero if it was previously out
; of range, and it's in both accumulator and temp.
;
LF34B
   lsr
   lsr
   lsr
   lsr
   tay
;
; Y=A/16 (Xcoord/16). Y = C in base-16.
;
   lda temp
   and #$0F
;
; A=A mod 16 (Xcoord mod 16). A = F in base-16.
;
   sty temp
   clc
   adc temp
; A = (Xcoord/16 + Xcoord mod 16)
; A now has the sum of the fine/coarse base-16 coordinates.
;
   cmp #$0F
   bcc LF360
   sbc #$0F
   iny
;
; The key section. The coarse coordinate is now in Y, and A has the fine
; coordinate (zero to 14). Y*15+A should equal the original X-coord (try
; tracing through the code with a few different values).
;
LF360
   cmp #XMIN
   eor #$0F
   bcs LF369
   adc #$01
   dey
;
; This converts the 0 to 14 setup of the fine coordinates to the +7 to -7
; setup that HMxx and HMOVE want. I think we increment Y here because we
; want to use a DEY/BNE loop, so zero iterations of the loop means Y should
; equal 1 (?)

LF369
   iny
   asl
   asl
   asl
   asl
   sta temp
   tya
   ora temp
;
; And this just puts the fine coarse together into the same 8-bit register.
;
   rts

;------------------------------------------------------------------RandomByte
;
; Again this routine comes courtesy of Andrew Davie. When this routine
; returns the accumulator will hold a random number. The random number bytes
; were purposly skipped during the clearing of the RAM so they would hold a
; random number each time the games was loaded into the 2600.
;
RandomByte
   lda random
   lsr
   lsr
   sbc random
   lsr
   ror random+1
   ror random
   ror random
   lda random
   rts
   
;------------------------------------------------------------------DrawScreen
;
; The 2600 has no screen RAM so the screen must be redrawn for each frame.
; Timing is very important. There are 76 machine cycles available for each
; scan line.
;
DrawScreen
   lda INTIM
   bne DrawScreen
   
   sta WSYNC                     ; end the current scan line
   sta VBLANK                    ; end the VBLANK period
   sta HMOVE
   
;   ldx tvType
;   lda FrameTime,x
;   sta T1024T
   
;   bit gameState
;   bmi TitleScreenKernel

   lda backgroundColor
   sta COLUBK
   jmp GameKernelStep

;----------------------------------------------------------------------Overscan
;
Overscan

   ldx tvType
   lda OverscanTime,x
   tax

	lda #$02
	sta WSYNC
	sta VBLANK
	stx TIM64T
	
OverscanLoop
   lda INTIM
   bne OverscanLoop
   
   rts
   
OverscanTime
   .byte #NTSCOVERSCANTIME
   .byte #PALOVERSCANTIME
   
   echo *-$F000, " bytes of code used"
   .org $F800
   
NumberFonts
   .byte $00;|        |
   .byte $3C;|  XXXX  |
   .byte $66;| XX  XX |
   .byte $66;| XX  XX |
   .byte $66;| XX  XX |
   .byte $66;| XX  XX |
   .byte $66;| XX  XX |
   .byte $3C;|  XXXX  |
   
   .byte $00;|        |
   .byte $3C;|  XXXX  |
   .byte $18;|   XX   |
   .byte $18;|   XX   |
   .byte $18;|   XX   |
   .byte $18;|   XX   |
   .byte $38;|  XXX   |
   .byte $18;|   XX   |
   
   .byte $00;|        |
   .byte $7E;| XXXXXX |
   .byte $60;| XX     |
   .byte $60;| XX     |
   .byte $3C;|  XXXX  |
   .byte $06;|     XX |
   .byte $46;| X   XX |
   .byte $3C;|  XXXX  |
   
   .byte $00;|        |
   .byte $3C;|  XXXX  |
   .byte $46;| X   XX |
   .byte $06;|     XX |
   .byte $0C;|    XX  |
   .byte $06;|     XX |
   .byte $46;| X   XX |
   .byte $3C;|  XXXX  |
   
   .byte $00;|        |
   .byte $0C;|    XX  |
   .byte $0C;|    XX  |
   .byte $7E;| XXXXXX |
   .byte $4C;| X  XX  |
   .byte $2C;|  X XX  |
   .byte $1C;|   XXX  |
   .byte $0C;|    XX  |
   
   .byte $00;|        |
   .byte $7C;| XXXXX  |
   .byte $46;| X   XX |
   .byte $06;|     XX |
   .byte $7C;| XXXXX  |
   .byte $60;| XX     |
   .byte $60;| XX     |
   .byte $7E;| XXXXXX |
   
   .byte $00;|        |
   .byte $3C;|  XXXX  |
   .byte $66;| XX  XX |
   .byte $66;| XX  XX |
   .byte $7C;| XXXXX  |
   .byte $60;| XX     |
   .byte $62;| XX   X |
   .byte $3C;|  XXXX  |
   
   .byte $00;|        |
   .byte $18;|   XX   |
   .byte $18;|   XX   |
   .byte $18;|   XX   |
   .byte $0C;|    XX  |
   .byte $06;|     XX |
   .byte $42;| X    X |
   .byte $7E;| XXXXXX |
   
   .byte $00;|        |
   .byte $3C;|  XXXX  |
   .byte $66;| XX  XX |
   .byte $66;| XX  XX |
   .byte $3C;|  XXXX  |
   .byte $66;| XX  XX |
   .byte $66;| XX  XX |
   .byte $3C;|  XXXX  |
   
   .byte $00;|        |
   .byte $3C;|  XXXX  |
   .byte $46;| X   XX |
   .byte $06;|     XX |
   .byte $3E;|  XXXXX |
   .byte $66;| XX  XX |
   .byte $66;| XX  XX |
   .byte $3C;|  XXXX  |
   
	IF DEBUGMODE

	.byte $00;|        |
   .byte $66;| XX  XX |
   .byte $66;| XX  XX |
   .byte $7E;| XXXXXX |
   .byte $66;| XX  XX |
   .byte $66;| XX  XX |
   .byte $66;| XX  XX |
   .byte $3C;|  XXXX  |

	.byte $00;|        |   
   .byte $7C;| XXXXX  |
   .byte $66;| XX  XX |
   .byte $66;| XX  XX |
   .byte $7C;| XXXXX  |
   .byte $66;| XX  XX |
   .byte $66;| XX  XX |
   .byte $7C;| XXXXX  |

	.byte $00;|        |   
	.byte $3C;|  XXXX  |
   .byte $66;| XX  XX |
   .byte $60;| XX     |
   .byte $60;| XX     |
   .byte $60;| XX     |
   .byte $66;| XX  XX |
   .byte $3C;|  XXXX  |

	.byte $00;|        |
	.byte $7C;| XXXXX  |
   .byte $66;| XX  XX |
   .byte $62;| XX   X |
   .byte $62;| XX   X |
   .byte $62;| XX   X |
   .byte $66;| XX  XX |
   .byte $7C;| XXXXX  |

	.byte $00;|        |
	.byte $7E;| XXXXXX |
   .byte $60;| XX     |
   .byte $60;| XX     |
   .byte $7E;| XXXXXX |
   .byte $60;| XX     |
   .byte $60;| XX     |
   .byte $7E;| XXXXXX |

	.byte $00;|        |
	.byte $60;| XX     |
   .byte $60;| XX     |
   .byte $60;| XX     |
   .byte $7E;| XXXXXX |
   .byte $60;| XX     |
   .byte $60;| XX     |
   .byte $7E;| XXXXXX |
   
	ENDIF
	
TitleData0
	.byte %01110011
   .byte %11111011
   .byte %11101011
   .byte %11101011
   .byte %11100011
   .byte %11101011
   .byte %11101011
   .byte %11111011
   .byte %01110011
      
TitleData1
   .byte %11101110
   .byte %10001110
   .byte %10001110
   .byte %10001110
   .byte %10001110
   .byte %10001110
   .byte %10001110
   .byte %10001110
   .byte %10001110
      
TitleData2
   .byte %11010101
   .byte %11010101
   .byte %11010101
   .byte %11010101
   .byte %11010101
   .byte %11010101
   .byte %11111101
   .byte %11111101
   .byte %01101001
      
TitleData3
   .byte %11100111
   .byte %11010111
   .byte %11010111
   .byte %11110111
   .byte %11100111
   .byte %11010111
   .byte %11010111
   .byte %11110111
   .byte %11100111
      
TitleData4
   .byte %11011101
   .byte %11011101
   .byte %11011101
   .byte %00011111
   .byte %10011110
   .byte %00011101
   .byte %11011101
   .byte %11011111
   .byte %11011110
      
BLANK
   .byte %00000000
   .byte %00000000
   .byte %00000000
   .byte %00000000   
   .byte %00000000
   .byte %00000000
   .byte %00000000
   .byte %00000000
   .byte %00000000

OddLadderTablePF1
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte %11000000
   .byte %11000000
   .byte %11000000
   .byte %11000000
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte %11000000
   .byte %11000000
   .byte %11000000
   .byte %11000000
   .byte $00
   .byte $00
   .byte $00
   .byte $00

OddLadderTablePF2
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte %00001100
   .byte %00001100
   .byte %00001100
   .byte %00001100
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte %00001100
   .byte %00001100
   .byte %00001100
   .byte %00001100
   .byte $00
   .byte $00
   .byte $00
   .byte $00

   
EvenLadderTablePF0
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte %00000110
   .byte %00000110
   .byte %00000110
   .byte %00000110
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte %00000110
   .byte %00000110
   .byte %00000110
   .byte %00000110
   .byte $00
   .byte $00
   .byte $00
   .byte $00

EvenLadderTablePF1
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte %00000110
   .byte %00000110
   .byte %00000110
   .byte %00000110
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte %00000110
   .byte %00000110
   .byte %00000110
   .byte %00000110
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   
EvenLadderTablePF2
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte %10000000
   .byte %10000000
   .byte %10000000
   .byte %10000000
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte %10000000
   .byte %10000000
   .byte %10000000
   .byte %10000000
   .byte $00
   .byte $00
   .byte $00
   .byte $00
         
HorizontalMoveFrame1   
	.byte $00,$00
	.byte $1C ;|   XXX  |
	.byte $3E ;|  XXXXX |   
	.byte $FE ;|XXXXXXX |
	.byte $3E ;|  XXXXX |
	.byte $5F ;| X XXXXX|
	.byte $7F ;| XXXXXXX|
	.byte $7F ;| XXXXXXX| mustache
	.byte $3E ;|  XXXXX | chin
	.byte $1C ;|   XXX  | neck
	.byte $3F ;|  XXXXXX|
	.byte $F9 ;|XXXXX  X|
	.byte $3A ;|  XXX X |
	.byte $32 ;|  XX  X |
	.byte $3E ;|  XXXXX |
	.byte $36 ;|  XX XX |
	.byte $36 ;|  XX XX |
	.byte $12 ;|   X  X |
	.byte $36 ;|  XX XX |

HorizontalMoveFrame2   
	.byte $00,$00
	.byte $1C ;|   XXX  |
	.byte $3E ;|  XXXXX |   
	.byte $FE ;|XXXXXXX |
	.byte $3E ;|  XXXXX |
	.byte $5F ;| X XXXXX|
	.byte $7F ;| XXXXXXX|
	.byte $7F ;| XXXXXXX| mustache
	.byte $3E ;|  XXXXX | chin
	.byte $1C ;|   XXX  | neck
	.byte $3E ;|  XXXXX |
	.byte $2B ;|  X X XX|
	.byte $23 ;|  X   XX|
	.byte $72 ;| XXX  X |
	.byte $7E ;| XXXXXX |
	.byte $6E ;| XX XXX |
	.byte $6F ;| XX XXXX|
	.byte $27 ;|  X  XXX|
	.byte $61 ;| XX    X|

HorizontalMoveFrame3	
	.byte $00,$00
	.byte $1C ;|   XXX  |
	.byte $3E ;|  XXXXX |   
	.byte $FE ;|XXXXXXX |
	.byte $3E ;|  XXXXX |
	.byte $5F ;| X XXXXX|
	.byte $7F ;| XXXXXXX|
	.byte $7F ;| XXXXXXX| mustache
	.byte $3E ;|  XXXXX | chin
	.byte $9C ;|X  XXX  | neck
	.byte $FF ;|XXXXXXXX|
	.byte $FF ;|XXXXXXXX|
	.byte $3A ;|  XXX X |
	.byte $76 ;| XXX XX |
	.byte $7E ;| XXXXXX |
	.byte $2F ;| X XXXX |
	.byte $66 ;| XX  XX |
	.byte $07 ;|     XXX|
	.byte $03 ;|      XX|
	
VerticalMoveFrame1
	.byte $00,$00
	.byte $1C ;|   XXX  | GREEN
	.byte $3E ;|  XXXXX | GREEN
   .byte $3E ;|  XXXXX | GREEN
   .byte $3E ;|  XXXXX | GREEN
   .byte $BE ;|X XXXXX | PEACH
   .byte $DC ;|XX XXX  | PEACH neck
   .byte $FE ;|XXXXXXX | GREEN
   .byte $73 ;| XXX  XX| GREEN
   .byte $37 ;|  XX XXX| GREEN
   .byte $13 ;|   X  XX| GREEN
   .byte $7B ;| XXXX XX| GREEN
   .byte $F2 ;|XXXX  X | GREEN
   .byte $FE ;|XXXXXXX | GREEN
   .byte $FE ;|XXXXXXX | GREEN
   .byte $FE ;|XXXXXXX | GREEN
   .byte $6E ;| XX XXX | GREEN
   .byte $0E ;|    XXX | GREEN
   .byte $04 ;|     X  | GREEN
	
VerticalMoveFrame2
	.byte $00,$00
	.byte $38 ;|  XXX   | GREEN
	.byte $7C ;| XXXXX  | GREEN
   .byte $7C ;| XXXXX  | GREEN
   .byte $7C ;| XXXXX  | GREEN
   .byte $7D ;| XXXXX X| PEACH
   .byte $3B ;|  XXX XX| PEACH neck
   .byte $7F ;| XXXXXXX| GREEN
   .byte $CE ;|XX  XXX |
   .byte $DC ;|XX XXX  | GREEN
   .byte $C8 ;|XX  X   | GREEN
   .byte $EE ;|XXX XXX | GREEN
   .byte $4F ;| X  XXXX| GREEN
   .byte $7F ;| XXXXXXX| GREEN
   .byte $7F ;| XXXXXXX| GREEN
   .byte $7F ;| XXXXXXX| GREEN
   .byte $76 ;| XXX XX | GREEN
   .byte $70 ;| XXX    | GREEN
   .byte $20 ;|  X     | GREEN
   
PlayerColor
;HorizontalColor
	.byte #BLACK, #BLACK
	.byte #GREEN
	.byte #GREEN
	.byte #GREEN
	.byte #PEACH
	.byte #PEACH
	.byte #PEACH
	.byte #PEACH
	.byte #PEACH
	.byte #PEACH
	.byte #GREEN
	.byte #GREEN
	.byte #GREEN
	.byte #GREEN
	.byte #GREEN
	.byte #GREEN
	.byte #GREEN
	.byte #GREEN
	.byte #LTBLUE
	
VerticalColor
	.byte #BLACK,#BLACK
	.byte #GREEN
	.byte #GREEN
   .byte #GREEN
   .byte #GREEN
   .byte #PEACH
   .byte #PEACH; neck
   .byte #GREEN
   .byte #GREEN
   .byte #GREEN
   .byte #GREEN
   .byte #GREEN
   .byte #GREEN
   .byte #GREEN
   .byte #GREEN
   .byte #GREEN
   .byte #GREEN
   .byte #GREEN
   .byte #LTBLUE
   
   echo *-$F800, "BYTES OF DATA USED"
   .org $FFFC
   
   .word Start
   .word Start